#include "PrivatWindow.h"
#include <ScrollView.h>
#include <TextControl.h>

#include "Utils.h"
#include "irc_view.h"
#include "StatusBarView.h"
#include "ServerWindow.h"
#include "PrivatView.h"
#include <stdio.h>
#include <string.h>
#include "constants.h"


//////////////////////////////////////
//int d_strnicmp(const char*str1, const char*str2);
void d_tolower(char *str);

//////////////////////////
///////
void PrivatWindow::internal_init( const char *name )
{
	BRect rc = Bounds(); 
	rc.bottom -= B_V_SCROLL_BAR_WIDTH; 
  	new PrivatView(this, rc);
  	rc.left--;
	rc.top = rc.bottom + 1;
	rc.bottom += B_V_SCROLL_BAR_WIDTH + 1;
	rc.right -= B_V_SCROLL_BAR_WIDTH - 0;
	StatusBarView *sbv = new StatusBarView(this, rc);
	sbv->SetResizingMode(B_FOLLOW_LEFT_RIGHT | B_FOLLOW_BOTTOM); 
	char newTitle[128];
	strcpy(m_name, name);	
	if( m_dcc )
		sprintf(newTitle, "DCC chat with <%s>", name );	
	else
		sprintf(newTitle, "Private chat with <%s>", name );	
		
	SetTitle(newTitle);	
	
}
PrivatWindow::PrivatWindow(ServerWindow* sw, const char* name) 
	: BWindow(BRect(30, 30, 380, 200), name, 
        B_DOCUMENT_WINDOW,
        /*B_NOT_ZOOMABLE|B_OUTLINE_RESIZE| B_FRAME_EVENTS*/B_ASYNCHRONOUS_CONTROLS
        ) 
{
	m_sw = sw;
	m_dcc = NULL;
 	internal_init( name );

	tag_wnd_item *item = new tag_wnd_item;
	item->pWnd = this;
 	strcpy( item->name, name);
	if( m_sw )
		m_sw->m_privat_list.AddItem((void*)item);
		
 	Show();
}
PrivatWindow::PrivatWindow(BHandler* dcc_handler, const char *name): BWindow(BRect(30, 30, 380, 200), name, 
        B_DOCUMENT_WINDOW,
        /*B_NOT_ZOOMABLE|B_OUTLINE_RESIZE| B_FRAME_EVENTS*/0
        ) 
{
	m_sw = NULL;
	m_dcc = dcc_handler;
 	AddHandler(m_dcc);
 	internal_init( name );	
 	Show();
}
void PrivatWindow::QuitBeforeSW(void)
{
	if( m_dcc )
	{
		RemoveHandler( m_dcc );
		delete m_dcc;
		m_dcc = NULL;
	}
	if( !m_sw)
		return;
	m_sw->Lock();
	tag_wnd_item* item = find_item_by_wnd( &m_sw->m_privat_list, this);
	m_sw->m_privat_list.RemoveItem( (void*)item );
	delete item;
	m_sw->Unlock();
	m_sw = NULL;
}
bool PrivatWindow::QuitRequested()
{
	QuitBeforeSW();
	if( m_dcc )
	{
		RemoveHandler( m_dcc );
		delete m_dcc;
		m_dcc = NULL;
	}
	return true;
}

void PrivatWindow::w_print(const char* str, int idx)
{
	if(!str)
		return;
	
	int32 len_str = strlen (str) + 1;
	if(len_str < 1)
	{
		return;
	}
	IRCView* irc_txt = (IRCView*)FindView("IRCView");
	if ( !irc_txt)
	{
		return;
	}
	irc_txt->w_print(str,  idx );
}
void PrivatWindow::do_msg(BMessage* message)
{

	const char *from = NULL;
	const char *command = NULL;
	const char *c_params = NULL;
	char *params = NULL;
	
	char sz_msg[1024];
	
	//PrivatView* pv = (PrivatView*)FindView("PrivatView");
		
	message->FindString ("params", &c_params);
	if(c_params)
	{	
		params = strdup( c_params );
		int len = strlen(params);
		if ( params[ len - 1] == '\n')
			params[ len - 1] = '\0';
	}
	message->FindString ("from", &from);
	
	message->FindString ("command", &command);
	if ( command && m_sw )
	{
		if( strcmp( command, "PRIVMSG" ) == 0 && params)
		{
			char *ptr = strchr( params, ':');
			if ( *(ptr + 1) == '\1')
			{
				if( !strncmp( ptr+2, "ACTION", 6) )
				{
					if ( !strncasecmp( params, m_sw->get_nick(), strlen(m_sw->get_nick()) ) )
					{	sprintf(sz_msg, "ACTION <%s> %s\n", from,  ptr + 9);
						w_print( sz_msg, eAction );
					}
				}
			}
			else
			{
				sprintf(sz_msg, "<%s> %s\n", from,  ptr + 1);
				//w_print( ptr + 1,  0);
				w_print( sz_msg, eNormal_Txt );
			}
		}
		if( strcmp( command, "PART") == 0 || strcmp( command, "QUIT") == 0 )
		{
			sprintf(sz_msg, "%s %s %s\n", command, from, params);
			w_print( sz_msg, eQuit);				
		}
			
		if( strcmp( command, "NICK") == 0)
		{
			m_sw->Lock();
			tag_wnd_item* item = find_item_by_wnd(&m_sw->m_privat_list, this);
			strcpy(item->name, params + 1);
			m_sw->Unlock();
			
			strcpy(m_name, params + 1);
			char newTitle[128];
			sprintf(newTitle, "Private with <%s>", m_name );
			SetTitle(newTitle);
			sprintf(sz_msg, "%s %s %s\n", from, command,  params + 1);
			w_print( sz_msg, eNick);		
		}
	}	
	if ( params ) 
		delete ( params );
}

void PrivatWindow::MessageReceived(BMessage* message)
{
	switch(message->what)
	{
		case 'qdcc':
		{
			RemoveHandler( m_dcc );
			delete m_dcc;
			m_dcc = NULL;
			w_print( "Disconnected\n", eError_Message );	
		}break;
		case 'dcc':
			{
				const char *sz_text;
				message->FindString ("text", &sz_text );
				w_print( "<", eError_Message );
				w_print( m_name, eNick );
				w_print( "> ", eError_Message );
				w_print( sz_text, eNormal_Txt );
			}
			break;
		case 'irc':
			do_msg(message);
			break;
		case 'ntr':
		{
			BMessage *l_target;
			if( message->FindPointer("target", (void**)&l_target) != B_OK)
				break;
			switch( l_target->what)
			{
				case 'umg':
				{	
					char priv_msg[600];
					BTextControl* mms = (BTextControl*)FindView("message");
					const char* sz_txt =  mms->Text();
					BMessage msg('irc');
					if( m_sw )
					{
						if (sz_txt[0] == '/')
						{
							if( strncasecmp( sz_txt, "/ME ", 4) == 0)
							{
								sprintf( priv_msg, "/ME %s %s", m_name, sz_txt + 4);
								msg.AddString("cmd", priv_msg );
								sprintf( priv_msg, "ACTION <%s> %s\n", m_sw->get_nick(),  sz_txt + 4);
								w_print( priv_msg, eAction  );
							}
							else 
								msg.AddString("cmd", sz_txt);
						}
						else
						{						
							sprintf(priv_msg, "/MSG %s %s", m_name, sz_txt);
							msg.AddString("cmd", priv_msg);
						}
						m_sw->msg_to_irc(&msg);					
						sprintf(priv_msg, "<%s> %s\n",  m_sw->get_nick(), sz_txt);
					}						
					if( m_dcc )
					{
						msg.AddString("cmd", sz_txt);
						PostMessage( &msg, m_dcc );						
						w_print( "> ", eError_Message );
						sprintf(priv_msg, "%s\n", sz_txt );
					}	 
					
					w_print(priv_msg, eOwn_Txt);							

					mms->SetText(NULL);
				}break;						
			}
		}
		break;	
	}
	BWindow::MessageReceived(message);
}

