#include "IRCApp.h"
#include "constants.h"
#include <Window.h>
#include <ScrollView.h>
#include <StatusBar.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <StringView.h>

#include "ServerView.h"
#include "Utils.h"
#include "irc.h"
#include "textFilter.h"
#include "myText.h"
#include "Prefs.h"
#include "irc_view.h"
#include "newItem.h"
#include <stdio.h>
#include <string.h>

//#include "ChatView.h"
#include "chanels_view.h"


//#define LH_TXT	30
ServerView::ServerView(BWindow* window, BRect rc)
	: BView(rc, "ServerView", B_FOLLOW_ALL, B_WILL_DRAW /*| B_NAVIGABLE_JUMP*/|B_FRAME_EVENTS)
{
	window->AddChild(this);
  //SetViewColor(116,116,116, 0);
	Prefs* prefs = ((IRCApp*)be_app)->GetPrefs();
	prefs->Lock();
	int16 chats_len = 180;
	prefs->GetMsg()->FindInt16("ChanelsListSize", &chats_len );
	prefs->Unlock();
  		
	rc = Bounds();
	rc.top ++;
	rc.right -= (chats_len + 6);
	IRCView *pIrc = new IRCView(rc);
	AddChild(pIrc);
	rc.left = rc.right + 6;
	rc.right = Bounds().right;
	chanels_view *pcv = new chanels_view(window, rc);
	AddChild( pcv );
	m_list_collapsed = false; 
	m_list_cash = NULL;	
}
ServerView::~ServerView()
{
	//RemoveAll();
}

void ServerView::ActivateProgress(int32 max_count)
{
	BStatusBar *stb = (BStatusBar*)FindView("progress");
	stb->Reset();
	stb->SetMaxValue(1000);
	
	
	if( max_count > 0 )
	{
	 	//ChatInfoView* ci = (ChatInfoView*)FindView("vChatInfo");
	 	BStringView *ci = (BStringView*)FindView("channels_info");
	 	
	 	//ci->SetString( "Count : 0" );
	 	ci->SetText("channel(s) : 0" );
	 	
	 	delete m_list_cash;
	 	m_list_cash = new BList( 100 );
	}
	else
	{
		BListView* ptr = (BListView*)FindView("channel_list");
		BLocker *ptr_lock = dynamic_cast<BLocker*>(ptr);
		ptr_lock->Lock();
		//ptr->AddItem( new chanelItem( str ));
		if( m_list_cash ) 
			ptr->AddList( m_list_cash );
		ptr_lock->Unlock();		
		delete m_list_cash;
		m_list_cash = NULL;
	}
	
}

void ServerView::AttachedToWindow(void)
{	
	SETUP_VIEW_COLORS;
	
	BView::AttachedToWindow();
	
}
void ServerView::DetachedFromWindow(void) 
{
	Prefs* prefs = ((IRCApp*)be_app)->GetPrefs();
	prefs->Lock();
	BView* ptr = FindView("_channels_view_");
	BRect rc = ptr->Bounds();
	int16 len = (int16)rc.Width();
	if( len < 0 ) len *= -1;//(int16)B_H_SCROLL_BAR_HEIGHT;
	prefs->GetMsg()->ReplaceInt16("ChanelsListSize", len );

	bool b = IsListCollapsed();
	prefs->GetMsg()->ReplaceBool("CollapseList", b );	
	
	prefs->Unlock();
	RemoveAll();
}
void ServerView::FrameResized(float width, float height)
{
	BView* ptr = FindView("IRCView");
	BRect rc( ptr->Bounds().RightTop(), BPoint(ptr->Bounds().RightBottom()) );
	rc.right += B_H_SCROLL_BAR_HEIGHT;
	Invalidate(rc);
}
void ServerView::AddItem(char *str)
{
	BWindow *w = Window();
	if(w) w->Lock();

	BStatusBar *stb = (BStatusBar*)FindView("progress");
	stb->Update(1.0);
	if( !m_list_cash )
		m_list_cash = new BList( 100 );		
		
	m_list_cash->AddItem( new chanelItem( str ) );
	if( !((int32)stb->CurrentValue()%100) )
	{
		BListView* ptr = (BListView*)FindView("channel_list");
		BLocker *ptr_lock = dynamic_cast<BLocker*>(ptr);
		ptr_lock->Lock();
		//ptr->AddItem( new chanelItem( str ));
		ptr->AddList(m_list_cash);
		ptr_lock->Unlock();
		delete m_list_cash;
	 	m_list_cash = new BList( 100 );		
	}
	char sz[ 64 ];	
	sprintf( sz,"channel(s) : %i", (int)stb->CurrentValue() );
	//ChatInfoView* ci = (ChatInfoView*)FindView("vChatInfo");
	//ci->SetString( sz );
	BStringView *ci = (BStringView*)FindView("channels_info");
 	ci->SetText( sz );
	
	if( stb->CurrentValue() >= stb->MaxValue() )
	{
		//stb->Reset();
		stb->SetMaxValue( stb->CurrentValue() + 1000 );
	}

	if(w)w->Unlock();
}
void ServerView::RemoveAll(void)
{
	BWindow *w = Window();
	if(w) w->Lock();
	BListView* ptr = (BListView*)FindView("channel_list");
	BLocker *ptr_lock = dynamic_cast<BLocker*>(ptr);
	ptr_lock->Lock();
	int32 count = ptr->CountItems();
	
	if( count )
	{
		const BListItem **ppList = ptr->Items();
		const BListItem **myList;
		myList = (const BListItem**)new BListItem*[count];
		int32 i = 0;
		for(  i = 0; i < count; i++)
			myList[i] = ppList[i];	
		ptr->MakeEmpty();		
		for(  i = 0; i < count; i++)
			delete myList[i];
		delete myList;
	}	

	delete m_list_cash;
	m_list_cash = NULL;
	ptr_lock->Unlock();
	if(w)w->Unlock();
}
void ServerView::CollapseList( bool b)
{
	if( b == m_list_collapsed )
		return;
	BView* ptr = FindView( "IRCView" );
	BView* ptr2 = FindView( "_channels_view_" );
	BRect rc = FindView( "<->" )->Bounds();
	BRect rc2 = ptr2->Bounds();
	float len =  ( b ? 1 : -1 ) * ( rc2.Width() - 1 );
	len -= ( b ? 1 : -1 ) * ( rc.Width() - 6 ); 
	ptr->ResizeBy(  len , 0 ); 		
	m_list_collapsed = b;	
}
void ServerView::Draw(BRect updateRect)
{
	BView::Draw(updateRect);
	//
	BView* ptr = FindView("IRCView");
	BRect rc( ptr->Bounds().RightTop(), ptr->Bounds().RightBottom() );
	rc.right += 6;
	rc.OffsetBy( 0, 1 );
	if( m_list_collapsed )
	{
		BView* ptr2 = FindView( "<->" );
		BRect r = ptr2->Bounds();
		rc.top += r.Height() + 1;
		rc.right += ( r.Width() - 4 );
	}
	SetHighColor(sc_viewcolor);
	FillRect(rc);
	SetHighColor(sc_highcolor);
	rc.InsetBy(1,  0); 
	draw_3Drect(this, rc, false);
	if( m_list_collapsed )
	{
		rc.InsetBy( 0 ,  4); 
		rc.left += 3;
		StrokeLine( rc.LeftTop(), rc.LeftBottom(), B_SOLID_HIGH);
		rc.left ++;
		StrokeLine( rc.LeftTop(), rc.LeftBottom(), B_SOLID_LOW);		
		rc.left += 2;
		StrokeLine( rc.LeftTop(), rc.LeftBottom(), B_SOLID_HIGH);
		rc.left ++;
		StrokeLine( rc.LeftTop(), rc.LeftBottom(), B_SOLID_LOW);		
		rc.left += 2;
		StrokeLine( rc.LeftTop(), rc.LeftBottom(), B_SOLID_HIGH);
		rc.left ++;
		StrokeLine( rc.LeftTop(), rc.LeftBottom(), B_SOLID_LOW);		
	}	

}
void ServerView::MouseDown(BPoint point)
{
	BView* ptr = FindView("IRCView");
	BView* ptr2 = FindView("_channels_view_");
	BRect rc_left = ptr->Bounds();
	BRect rect(rc_left.RightTop(), rc_left.RightBottom() ); 
	rect.right += 6;
	BRect rc_border = Bounds();
	//rect.OffsetBy(1, 0);

	float startx = point.x;
	
	unsigned long buttons; 
	
	if( rect.Contains(point) && !m_list_collapsed )
	{
		do
		{ 
			snooze(80 * 1000); 
			GetMouse(&point, &buttons);
			if( point.x < rc_border.left )
				point.x = rc_border.left;
			 if( point.x > rc_border.right )
			 	point.x = rc_border.right;

			if(point.x != rc_left.right)
			{
					ptr->ResizeBy(point.x - startx, 0);//rc_left.Height());
					ptr2->ResizeBy(startx - point.x  , 0);
					ptr2->MoveBy(point.x - startx, 0);
					rect.OffsetBy(point.x - startx, 0); 
					Invalidate(rect);
					//Window()->UpdateIfNeeded();
					startx = point.x;
			}
			
	 	}
	 	while ( buttons );	 	
	}			

	BMessageQueue *mq = Window()->MessageQueue();
	mq->Lock();
	while( mq->CountMessages() > 14 )
	{
		BMessage *msg = mq->NextMessage();
		if( !msg )
			break;
		delete msg;		
	}		
	mq->Unlock();
	
	//ptr->Invalidate();
	//ptr2->Invalidate();

//	((IRCView*)ptr)->set_brodcast(true);	
	Window()->UpdateIfNeeded();
}

void ServerView::MessageReceived(BMessage* message)
{
  switch (message->what)
  {
	
	case 'mdn':
	{
		BListView* ptr = (BListView*)FindView("channel_list");
		BPopUpMenu* popup = new BPopUpMenu("server_menu", false);//, false );
		popup->SetFont(be_plain_font);
		BPoint pt;
		BMessage *item_msg = NULL;
		char sz_str[256];
		message->FindPoint("mouse", &pt);			
		BLocker *ptr_lock = dynamic_cast<BLocker*>(ptr);
		
		ptr_lock->Lock();
		int32 selected = ptr->CurrentSelection();
		ptr_lock->Unlock();
		
		if(selected >= 0)
		{
			item_msg = new BMessage('itm');
			ptr_lock->Lock();
			chanelItem* itemptr = (chanelItem*)ptr->ItemAt( selected );
			const char *name = itemptr->get_name();
			char *sz_tmp = strdup(name);
			ptr_lock->Unlock();
		
			char *end_ptr = strchr(sz_tmp, ' ');
			*end_ptr = '\0';
			sprintf(sz_str, "/JOIN %s", sz_tmp);
			item_msg->AddString( "cmd", sz_str);	
			sprintf(sz_str, "Join to %s", sz_tmp);
			delete sz_tmp;
			BMenuItem *item = new BMenuItem( sz_str, item_msg);
			popup->AddItem(item);		
		}
		
		item_msg = new BMessage('itm');
			
		sprintf(sz_str, "/LIST");
		item_msg->AddString( "cmd", sz_str);			
		sprintf(sz_str, "Reload channels list");
		BMenuItem *item = new BMenuItem( sz_str, item_msg);
		popup->AddItem(item);			
		
		//BMenuItem *item;
		BMenu* sort_popup = new BMenu("Sort by");
		sort_popup->SetFont(be_plain_font);
		
		item_msg = new BMessage('sort');
		item_msg->AddInt16( "sort_type", 0 );
		item = new BMenuItem( "channel", item_msg);
		sort_popup->AddItem(item);			

		item_msg = new BMessage('sort');
		item_msg->AddInt16( "sort_type", 1 );
		item = new BMenuItem( "users", item_msg);
		sort_popup->AddItem(item);			
		
		item_msg = new BMessage('sort');
		item_msg->AddInt16( "sort_type", 2 );
		item = new BMenuItem( "topic", item_msg);
		sort_popup->AddItem(item);			
		sort_popup->SetTargetForItems(Window());
		popup->AddItem( sort_popup );	
		
		ptr->ConvertToScreen(&pt);
		popup->SetTargetForItems(Window());
		BRect rc( pt.x - 2, pt.y - 2, pt.x + 2, pt.y + 2);
		popup->Go(pt, true, true, rc, true); 
		//delete popup;
			
	}
	break;
	
	default:    	
	break;
  }
	BView::MessageReceived(message);
}
