#include "Utils.h"
#include "StatusBarView.h"
//#include "ChatWindow.h"
#include "DirtyWnd.h"

#include <View.h>
#include <Window.h>
//#include <unistd.h>


static BBitmap *sbmp1 = NULL;
static BBitmap *sbmp2 = NULL;
static const int sc_dx = 50;

BBitmap *RetrieveBitmap( int32 res_id );

StatusBarView::StatusBarView(BWindow* window, BRect rc)
: BView(rc, "statusbar", B_FOLLOW_LEFT | B_FOLLOW_BOTTOM
						,  B_WILL_DRAW | B_PULSE_NEEDED )
{
	m_first.pWnd = NULL;
	//SetViewColor(212, 212, 212);
  	window->AddChild(this);
	//m_bDirty = true;
	if (!sbmp1)
		sbmp1 = RetrieveBitmap( 5 );
	if (!sbmp2)
		sbmp2 = RetrieveBitmap( 6 );
	m_o_list = NULL;	
	m_bDirty = false;
}
StatusBarView::~StatusBarView(void)
{
	m_lock.Lock();
	int32 i, count = m_list.CountItems();
	for( i = 0; i < count; i++)
	{
		delete (tag_item*)m_list.RemoveItem(int32(0));
	}
	m_lock.Unlock();
}
void StatusBarView::AttachedToWindow()
{
	const rgb_color	viewcolor	= {220, 220, 220};
	const rgb_color	highcolor	= {250, 250, 250};
	const rgb_color	lowcolor	= {150, 150, 150};
	
	this->SetViewColor(viewcolor);
	this->SetHighColor(highcolor);
	this->SetLowColor(lowcolor);

	BFont font (be_fixed_font) ;
	font.SetFlags(B_DISABLE_ANTIALIASING);
	font.SetSize(12 );
	SetFont(&font);
}

void StatusBarView::AddList(BList *list)
{
	m_lock.Lock();
	m_o_list = list;
	m_lock.Unlock();
}
void StatusBarView::AddFirst(BWindow *pWnd)
{ 
	m_lock.Lock();
	m_first.pWnd = pWnd;
	m_first.dirty = false;
	strcpy(m_first.name, "Server");
	m_lock.Unlock();
}
int32 StatusBarView::find_item(BWindow *ptr)
{
	if(!m_o_list)
		return -1;
	m_lock.Lock();
	int32 i = 0, count = m_list.CountItems();
	for( i = 0; i < count; i++)
	{
		tag_item *item = (tag_item*)m_list.ItemAt(i);
		if( item->pWnd == ptr)
		{
			m_lock.Unlock();
			return i;
		}
	}
	m_lock.Unlock();
	return -1;
}
void StatusBarView::check_window(void)
{
	m_bDirty = false;
	if(!m_o_list)
		return;
	if( Window() == NULL)	
		return;
	m_lock.Lock();
	//BList list;
	int32 i, count = m_o_list->CountItems();
	// add item
	
	
	for( i = 0; i < count; i++)
	{
	 	tag_wnd_item* ptr = (tag_wnd_item*)m_o_list->ItemAt( i );
	 	if( ptr )
	 	{
		 		int32 idx = find_item( ptr->pWnd );
		 		if( idx == -1 )
		 		{
		 			tag_item *item = new tag_item;
		 			item->pWnd = ptr->pWnd;
		 			strcpy( item->name, ptr->name);
		 			try
		 			{	
		 				item->dirty = dynamic_cast<DirtyWnd*>(ptr->pWnd)->is_dirty();
		 				m_list.AddItem( (void*)item);
		 			}
		 			catch(...)
		 			{
		 			}
		 			m_bDirty = true;
		 		}		 			 	
		}
	}//for
	//remove item
	count = m_list.CountItems();
	for( i = 0; i < count; i++)
	{
		tag_item *item = (tag_item*)m_list.ItemAt(i);
		if( !item )
			continue;
		tag_wnd_item* ptr = find_item_by_name(  m_o_list,  item->name);
		 if( !ptr )
		 {
		 	delete  (tag_item*)m_list.RemoveItem(i);
		 	m_bDirty = true;
		 	i--;
		 	count--;
		 }
		 else
		 {
		 	bool dirty;
		 	try
		 	{
		 		dirty = dynamic_cast<DirtyWnd*>(ptr->pWnd)->is_dirty();
		 		if ( dirty != item->dirty)
		 		{	
		 			item->dirty = dirty;
		 			m_bDirty = true;
		 		}
		 	}
		 	catch(...)
		 	{
		 		delete  (tag_item*)m_list.RemoveItem(i);
		 		m_bDirty = true;
		 		i--;
		 		count--;
		 	}
		 }
	}
	if (m_first.pWnd )
	{
		bool dirty = !m_first.dirty;
		try
		{
			dirty = dynamic_cast<DirtyWnd*>(m_first.pWnd)->is_dirty();
			if ( dirty != m_first.dirty)
		 	{	
		 		m_first.dirty = dirty;
		 		m_bDirty = true;
		 	}
		 }
		catch(...)
		{
			m_first.pWnd = NULL;
			m_bDirty = true;			
		}
		
	}

	m_lock.Unlock();
}

void StatusBarView::Draw(BRect updateRect)
{
	BRect frame;
	BRect	dRect(7, 3 , 9, 9);  
	dRect.OffsetBy(frame.LeftTop());

	
	frame = Bounds();
	int32 i = 0, count = m_list.CountItems();
	int x = 0;
	font_height height;
	GetFontHeight(&height);
	rgb_color color = HighColor();
	
	frame.top += 2;
	
	m_lock.Lock();
	count++;
	int32 real_i = 0;	
	for( i = 0; i < count; i++)
	{
		if(m_first.pWnd == NULL && i == 0 )
			continue;
		SetHighColor (ViewColor());	//SetHighColor (192, 192, 192);	
		FillRect( BRect( x, frame.top, x + sc_dx, frame.bottom)); 
		tag_item *item = NULL;
		if( i != 0 )
			item = (tag_item*)m_list.ItemAt( real_i );
		else
			item = &m_first;
		SetHighColor(color);
		StrokeLine(BPoint( x, frame.top + 0), BPoint( x, frame.bottom), B_SOLID_HIGH);	
		x++;
		MovePenTo( x , frame.top +  height.ascent + height.leading );	
		if ( item && item->dirty)
			SetHighColor (128, 0, 0);
		else
			SetHighColor (0, 0, 0);
				
		if( item )DrawString( item->name );
		//draw_frame(this, BRect(x, frame.top - 2,  x + sc_dx , frame.bottom), false); 

		x += sc_dx;
		//SetHighColor(color);
		//StrokeLine(BPoint( x, frame.top + 0), BPoint( x, frame.bottom), B_SOLID_LOW);
		//x++;
		StrokeLine(BPoint( x, frame.top + 0), BPoint( x, frame.bottom), B_SOLID_LOW);
		x++;
		if( i != 0)
			real_i++;
	}
	m_lock.Unlock();
	x++;	
	SetHighColor (ViewColor());	
	FillRect( BRect( x, frame.top, frame.right,  frame.bottom)); 
	SetHighColor(color);
	
	frame = Bounds();
	
	SetPenSize(1.0);
	//Upper dark gray line.
	MovePenTo(frame.LeftTop());
	StrokeLine(frame.RightTop(), B_SOLID_LOW);
	
	//Left light gray line.
	BPoint	point = frame.LeftTop();
	point.y += 1;
	MovePenTo(point);
	StrokeLine(frame.LeftBottom(), B_SOLID_HIGH);
	
	//Upper light gray line.
	BPoint	topoint = frame.RightTop();
	topoint.y += 1;
	MovePenTo(point);
	StrokeLine(topoint, B_SOLID_HIGH);

}
void StatusBarView::MouseDown(BPoint point)
{
	BView::MouseDown(point);
	int idx = int((point.x / (float)(sc_dx + 2)) - 0.0);
	
	m_lock.Lock();
		
	tag_item *item = NULL;
	if(m_first.pWnd == NULL)
		item = (tag_item*)m_list.ItemAt( idx );
	else
	{
		if( idx == 0)
		  item = &m_first;
		else
			item = (tag_item*)m_list.ItemAt( idx -1 );	
	}	
		
	if( !item )
	{
		m_lock.Unlock();
		return;
	}
	BRect rc(idx*(sc_dx + 2), 1, (idx + 1)*(sc_dx + 2) - 1, Bounds().Height());
	drawing_mode dm = DrawingMode();
	SetDrawingMode( B_OP_INVERT );
	FillRect( rc ); 
	Sync();
	item->pWnd->Activate(true);
	snooze(100 * 1000);		  
	FillRect( rc ); 
	SetDrawingMode( dm );
	m_lock.Unlock();
		
	//Invalidate( rc );

}
void StatusBarView::MessageReceived(BMessage* message)
{
      
	switch (message->what)
  	{    
    		default:      		
    		break;
  	}
	BView::MessageReceived(message);
}
void StatusBarView::Pulse()
{	
	//this->m_bDirty = !this->m_bDirty;
	check_window();
	if( m_bDirty )
		this->Invalidate();
	BView::Pulse();
}
//////////////////
///////
/*void StatusBarView::AddWindow(BWindow *pWnd)
{
 // LastItem();
 // list.AddItem(pWnd);
  
}
void StatusBarView::RemoveWindow(BWindow *pWnd)
{
 // list.RemoveItem(pWnd);
}
*/