#include <sys/types.h>
#include <ctype.h>
#include <netdb.h>
#include <string.h>
#include <stdio.h>

#include "socks5.h"

char socks5error[128];

bool socks5command( int s,  unsigned char cmd, const char * hostname, short port, const char* user, const char* password )
{
	strcpy( socks5error, "No Errors");
	unchar buf[ 128 ] = "\005\001\000";
	int buf_len = 3;
	if( user && password && user[ 0 ] && password[ 0 ] )
	{
		buf[ 1 ] = '\002';
		buf[ 3 ] = SOCKS_AUTH_USER_PASS;
		buf_len = 4;
	}
		
	int ret = send ( s , buf, buf_len, 0);
	if( ret < 0 )
	{
		strcpy( socks5error, "1, send() < 1");
		return false;
	}

	buf_len = 0;

while( 1 )	
{
	ret = recv( s , buf, 2,  0 );	
	if( ret != 2 )
	{
		strcpy( socks5error,  "1, reply() < 2");
		return false;
	}
	if( buf[1] == SOCKS_AUTH_FAILED )
	{
		sprintf( socks5error, "Authorization FAILED");
		return false;
	}
	if( buf[1] == SOCKS_AUTH_USER_PASS )
	{
		// Send username and pasword
		buf[ buf_len++ ] = '\01';
		unchar sz_len = (unchar)strlen( user );
		buf[ buf_len++ ] = sz_len;
		strcpy( (char*)&buf[ buf_len ], user);
		buf_len += sz_len + 0;
		
		sz_len = (unchar)strlen( password );
		buf[ buf_len++ ] = sz_len;
		strcpy( (char*)&buf[ buf_len ], password);
		buf_len += sz_len + 0;
		ret = send ( s , buf, buf_len, 0);
		if( ret < buf_len )
		{
			strcpy( socks5error,  "au, send < buf_len");
			return false;
		}
		continue;		
	}
	if( buf[1] == '\0' )
		break;
	sprintf( socks5error, "Authorization FAILED: %i", buf[1] );
	return false;
		
}	

	buf_len = 0;
	buf[ buf_len++ ] = SOCKS_VERSION_5;
	buf[ buf_len++ ] = cmd;
	buf[ buf_len++ ] = '\0'; //reserved	
	if (hostname != NULL)
	{
		unchar sz_len = (unchar)strlen(hostname);
		buf[ buf_len++ ] = SOCKS_ADDR_DOMAINNAME;
		buf[ buf_len++ ] = sz_len + 0;
		strcpy( (char*)&buf[ buf_len ], hostname);
		buf_len += sz_len + 0;
	}	
	else 
	{	
		buf[ buf_len++ ] = SOCKS_ADDR_IPV4;
		 *((ulong*)&buf[ buf_len]) = 0; 
		buf_len += 4; 
		
		//strcpy( socks5error,  "hostname == NULL");
		//return false;
	} 
	
	*((short*)&buf[ buf_len ])  = port;
	 buf_len += 2; 
	ret = send ( s , buf, buf_len, 0);
	if( ret < buf_len ) 
	{
		strcpy( socks5error,  "send < buf_len");
		return false;
	}
	ret = recv( s , buf, 2,  0 );
	if( ret != 2 )
	{
		strcpy( socks5error,  "2, reply != 2");
		return false;		
	}
	if( buf[1] != '\000' )
	{
		switch( buf[1] )
		{
			case '\02': strcpy( socks5error, "connection not allowed by ruleset");break;
			case '\03': strcpy( socks5error, "Network unreachable");break;
			case '\04': strcpy( socks5error, "Host unreachable");break;
			case '\05': strcpy( socks5error, "Connection refused");break;
			case '\06': strcpy( socks5error, "TTL expired");break;
			case '\07': strcpy( socks5error, "Command not supported");break;
			case '\08': strcpy( socks5error, "Address type not supported");break;
			case '\09': strcpy( socks5error, "to X'FF' unassigned");break;
		}
		return false;
	}

	
	ret = recv( s , buf, 1,  0 );// reserved byte
	if( ret != 1)
	{
		strcpy( socks5error,  "3, reply != 1");
		return false;		
	}
	
	// Get type byte for bound address
	ret = recv( s , buf, 1,  0 );
	if( ret != 1)	
	{
		strcpy( socks5error,  "4, reply != 1");
		return false;		
	}
	
	switch ( buf[0] ) 
	{
		case SOCKS_ADDR_DOMAINNAME :
			strcpy( socks5error,  "Not supported: SOCKS_ADDR_DOMAINNAME");
			return false;						

		case SOCKS_ADDR_IPV4 :
				ulong reply_addr;
				ret = recv( s , &reply_addr, sizeof(ulong), 0);
				if( ret != sizeof(ulong) )
				{
					strcpy( socks5error,  "5, reply != sizeof(ulong)" );
					return false;		
				}
				//printf("Ret Addr:\t%s\n", inet_ntoa(*(in_addr*)hp->h_addr));
				break;
		default :
      		strcpy( socks5error,  "Unknown Reply Command");
      		return false;	
	}

	short lPort;
	ret = recv( s , &lPort, sizeof(lPort), 0);
	if( ret != sizeof(short) )
	{
		strcpy( socks5error,  "6, reply != sizeof(short)" );
		return false;		
	}
	return true;
}
